@@ -11,6 +11,10 @@ require "eatmanager"
 include Gtk
 
 if __FILE__ == $0
+	if not GLib::Thread.supported?
+		GLib::Thread.init()
+	end
+
 	aria2 = Eat::Aria2Listener.instance
 	settings = Eat::Settings.instance
 	if settings["custom-server"]
