@@ -4,9 +4,11 @@ require "xmlrpc/client"
 require "fileutils"
 require "singleton"
 require "timeout"
+require "thread"
 require "pp"
 
 require "config"
+require "compat18"
 require "glib2"
 
 def debug(*args)
@@ -42,6 +44,8 @@ class Eat::Aria2 < GLib::Object
 			nil)					# Return type: void
 	signal_new("disconnected", GLib::Signal::RUN_FIRST, nil,
 			nil)					# Return type: void
+	signal_new("shutdown", GLib::Signal::RUN_FIRST, nil,
+			nil)					# Return type: void
 	signal_new("download_status", GLib::Signal::RUN_FIRST, nil,
 			nil,					# Return type: void
 			String)					# Parameters: gid
@@ -59,6 +63,8 @@ class Eat::Aria2 < GLib::Object
 	end
 	def signal_do_disconnected()
 	end
+	def signal_do_shutdown()
+	end
 	def signal_do_download_status(gid)
 	end
 	def signal_do_download_completed(gid)
@@ -82,6 +88,7 @@ class Eat::Aria2 < GLib::Object
 		@use_local_server = true
 		@version = "n/a"
 		@@client = nil if !defined? @@client
+		@shutdown_thread = nil
 
 		if !defined? @@pid
 			begin
@@ -117,24 +124,22 @@ class Eat::Aria2 < GLib::Object
 				# Cleanup log file
 				FileUtils.rm(@confdir+"/aria2.log", :force => true)
 				# Launch aria2c process
-				command = "aria2c --disable-ipv6=true " \
-					"--enable-xml-rpc --xml-rpc-listen-port=#{@port} " \
-					"--conf-path=#{@confdir}/aria2.conf --log=#{@confdir}/aria2.log --log-level=notice " \
-					"--dht-file-path=#{@confdir}/dht.dat"
-				debug("start server", command)
-				@@pid = Process.spawn(command, :pgroup=>true, :chdir=>ENV['HOME'],
-						STDOUT=>"/dev/null", STDIN=>"/dev/null")
-				Process.detach(@@pid)
-				# Wait for the server to respond properly to requests
-				Timeout::timeout(5) do
-					begin
-						debug("test request")
-						begin
-							result = @@client.call("aria2.getVersion")
-						rescue
-						end
-						sleep 1 if result == nil
-					end while result == nil
+				begin
+					command = "aria2c --disable-ipv6=true " \
+						"--enable-xml-rpc --xml-rpc-listen-port=#{@port} " \
+						"--conf-path=#{@confdir}/aria2.conf --log=#{@confdir}/aria2.log --log-level=notice " \
+						"--dht-file-path=#{@confdir}/dht.dat"
+					@@pid = spawn_server(command)
+				rescue Timeout::Error
+				begin
+					# Retry with fewer arguments for aria2 < 1.7
+					command = "aria2c --enable-xml-rpc --xml-rpc-listen-all --xml-rpc-listen-port=#{@port} " \
+						"--conf-path=#{@confdir}/aria2.conf --log=#{@confdir}/aria2.log --log-level=notice"
+					@@pid = spawn_server(command)
+				rescue Timeout::Error
+					@@pid = 0
+					raise "Failed to get positive response from aria2"
+				end
 				end
 				# Store pid in a file
 				begin
@@ -152,6 +157,25 @@ class Eat::Aria2 < GLib::Object
 		end
 	end
 
+	def spawn_server(command)
+		pid = Process.spawn(command, :pgroup=>true, :chdir=>ENV['HOME'],
+				STDOUT=>"/dev/null", STDIN=>"/dev/null")
+		Process.detach(pid)
+		# Wait for the server to respond properly to requests
+		debug("waiting for a first response...")
+		Timeout::timeout(5) do
+			begin
+				begin
+					result = @@client.call("aria2.getVersion")
+				rescue
+				end
+				sleep 1 if result == nil
+			end while result == nil
+		end
+		debug("started server", command, pid)
+		pid
+	end
+
 	def call(method, *args)
 		begin
 			result = @@client.call(method, *args)
@@ -190,20 +214,23 @@ class Eat::Aria2 < GLib::Object
 	def connect(force=false)
 		return if @@client and force == false
 		@is_connected = false
-		begin
-			debug("connect")
-			@@client = XMLRPC::Client.new3({:host => @hostname, :path => "/rpc",
-					:port => @port, :user => @user, :password => @password,
-					:timeout => 60})
-			result = call("aria2.getVersion")
-			raise Exception.new("Unable to connect to aria2 XML-RPC server") if result == nil
-			@version = result["version"]
-			@is_connected = true
-			signal_emit("connected")
-		rescue Exception => e
-			@@client = nil
-			puts "Can't establish a connection"
-			debug(e.message)
+		Thread.new do
+			@shutdown_thread.join if @shutdown_thread != nil
+			begin
+				debug("connect")
+				@@client = XMLRPC::Client.new3({:host => @hostname, :path => "/rpc",
+						:port => @port, :user => @user, :password => @password,
+						:timeout => 60})
+				result = call("aria2.getVersion")
+				raise Exception.new("Unable to connect to aria2 XML-RPC server") if result == nil
+				@version = result["version"]
+				@is_connected = true
+				signal_emit("connected")
+			rescue Exception => e
+				@@client = nil
+				puts "Can't establish a connection"
+				debug(e.message)
+			end
 		end
 	end
 
@@ -240,21 +267,24 @@ class Eat::Aria2 < GLib::Object
 	Disconnect and terminates the local server if it exists.
 =end
 	def shutdown()
-		debug("shutdown", @@pid)
-		begin
-			if @@pid > 0
-				FileUtils.rm(@confdir+"/aria2.pid", :force => true)
-				Process.kill("TERM", @@pid)
-				sleep 1 while Process.kill("CONT", @@pid)
+		@shutdown_thread = Thread.new do
+			debug("shutdown", @@pid)
+			@is_connected = false
+			begin
+				if @@pid > 0
+					FileUtils.rm(@confdir+"/aria2.pid", :force => true)
+					Process.kill("TERM", @@pid)
+					sleep 1 while Process.kill("CONT", @@pid)
+				end
+			rescue Errno::ESRCH
+				# Process does no more exist
+			rescue
+				p $!
 			end
-		rescue Errno::ESRCH
-			# Process does no more exist
-		rescue
-			p $!
+			@@pid = 0
+			@@client = nil
+			signal_emit("shutdown")
 		end
-		@@pid = 0
-		@@client = nil
-		@is_connected = false
 	end
 
 	# Adds new HTTP(S)/FTP/BitTorrent/Magnet URI.
