@@ -45,13 +45,14 @@ class Eat::Manager
 		@action_pause = builder["action-pause"]
 		@action_resume = builder["action-resume"]
 		@action_remove = builder["action-remove"]
+		@action_settings = builder["action-settings"]
 
 		# Setup actions
 		@action_add.signal_connect("activate") { action_add }
 		@action_pause.signal_connect("activate") { action_pause }
 		@action_resume.signal_connect("activate") { action_resume }
 		@action_remove.signal_connect("activate") { action_remove }
-		builder["action-settings"].signal_connect("activate") { show_settings_dialog }
+		@action_settings.signal_connect("activate") { show_settings_dialog }
 		builder["action-quit"].signal_connect("activate") { action_quit }
 		builder["action-about"].signal_connect("activate") { show_about_dialog }
 
@@ -220,6 +221,7 @@ class Eat::Manager
 		if res == Dialog::RESPONSE_OK
 
 			settings = Eat::Settings.instance
+			set_sensitive(false)
 
 			if settings["custom-server"]
 				# Switch to custom server
@@ -263,8 +265,11 @@ class Eat::Manager
 	private
 
 	def action_quit()
-		@aria2.shutdown # TODO make this optional
-		main_quit
+		# TODO make this optional e.g. keep active downloads running
+		set_sensitive(false)
+		@action_settings.set_sensitive(false)
+		@aria2.signal_connect("shutdown") { main_quit }
+		@aria2.shutdown
 	end
 
 	def action_add()
